#ifndef CHOICECONDITION_H_
#define CHOICECONDITION_H_

#include <msp/core/algorithm.h>
#include "condition.h"
#include "filter.h"

template<typename T>
class ChoiceCondition: public Condition
{
public:
	typedef T Traits;
	typedef typename Traits::Type Type;

private:
	std::vector<Type> values;

public:
	ChoiceCondition(Type);
	ChoiceCondition(const std::vector<Type> &);

	virtual ChoiceCondition<T> *clone() const;
	virtual bool equals(const Condition &) const;
	virtual bool can_merge(const Condition &, const CompoundCondition &) const;
	virtual ChoiceCondition<T> *merge(const std::vector<const Condition *> &, const CompoundCondition &) const;
	virtual void add_lines(std::list<FilterStatement> &) const;
};


struct ClassTraits
{
	typedef std::string Type;
	static const char *get_keyword() { return "Class"; }
};

typedef ChoiceCondition<ClassTraits> ClassCondition;


struct BaseTypeTraits
{
	typedef std::string Type;
	static const char *get_keyword() { return "BaseType"; }
};

typedef ChoiceCondition<BaseTypeTraits> BaseTypeCondition;


template<typename T>
ChoiceCondition<T>::ChoiceCondition(Type v)
{
	values.push_back(v);
}

template<typename T>
ChoiceCondition<T>::ChoiceCondition(const std::vector<Type> &v):
	values(v)
{ }

template<typename T>
ChoiceCondition<T> *ChoiceCondition<T>::clone() const
{
	return new ChoiceCondition<T>(values);
}

template<typename T>
bool ChoiceCondition<T>::equals(const Condition &other) const
{
	const ChoiceCondition<T> *other_choice = dynamic_cast<const ChoiceCondition<T> *>(&other);
	if(!other_choice || values.size()!=other_choice->values.size())
		return false;

	for(unsigned i=0; i<values.size(); ++i)
		if(values[i]!=other_choice->values[i])
			return false;

	return true;
}

template<typename T>
bool ChoiceCondition<T>::can_merge(const Condition &other, const CompoundCondition &parent) const
{
	return dynamic_cast<const ChoiceCondition<T> *>(&other) && dynamic_cast<const OrCondition *>(&parent);
}

template<typename T>
ChoiceCondition<T> *ChoiceCondition<T>::merge(const std::vector<const Condition *> &conditions, const CompoundCondition &parent) const
{
	if(dynamic_cast<const OrCondition *>(&parent) && !conditions.empty())
	{
		ChoiceCondition<T> *result = new ChoiceCondition<T>(std::vector<Type>());
		for(std::vector<const Condition *>::const_iterator i=conditions.begin(); i!=conditions.end(); ++i)
		{
			const std::vector<Type> &v = static_cast<const ChoiceCondition<T> *>(*i)->values;
			for(typename std::vector<Type>::const_iterator j=v.begin(); j!=v.end(); ++j)
				if(Msp::find(result->values, *j)==result->values.end())
					result->values.push_back(*j);
		}
		return result;
	}
	else
		return 0;
}

template<typename T>
void ChoiceCondition<T>::add_lines(std::list<FilterStatement> &st) const
{
	std::string line = Traits::get_keyword();
	for(typename std::vector<Type>::const_iterator i=values.begin(); i!=values.end(); ++i)
		line += Msp::format(" \"%s\"", *i);
	FilterStatement::add_line(st, line);
}

#endif
