#include <msp/core/getopt.h>
#include <msp/core/maputils.h>
#include <msp/io/file.h>
#include "category.h"
#include "filter.h"
#include "poefilter.h"

using namespace std;
using namespace Msp;

PoeFilter::PoeFilter(int argc, char **argv)
{
	GetOpt getopt;
	getopt.add_argument("filename", filename, GetOpt::REQUIRED_ARG);
	getopt(argc, argv);
}

int PoeFilter::main()
{
	DataFile::load(*this, filename);

	for(map<string, Filter>::const_iterator i=filters.begin(); i!=filters.end(); ++i)
		if(!i->second.is_abstract())
		{
			IO::BufferedFile out(i->first+".filter", IO::M_WRITE);
			i->second.write(out);
		}

	return 0;
}

const Category &PoeFilter::get_category(const string &name) const
{
	return get_item(categories, name);
}

void PoeFilter::find_categories(const Regex &re, vector<const Category *> &categs) const
{
	for(map<string, Category>::const_iterator i=categories.begin(); i!=categories.end(); ++i)
		if(re.match(i->first))
			categs.push_back(&i->second);
}

const Filter &PoeFilter::get_filter(const string &name) const
{
	return get_item(filters, name);
}


PoeFilter::Loader::Loader(PoeFilter &f):
	DataFile::ObjectLoader<PoeFilter>(f)
{
	add("category", &Loader::category);
	add("filter", &Loader::filter);
	add("import", &Loader::import);
	add("theme", &Loader::theme);
}

void PoeFilter::Loader::category(const string &name)
{
	Category cat(name, obj.categories.size());
	load_sub(cat, obj);
	obj.categories[name] = cat;
}

void PoeFilter::Loader::filter(const string &name)
{
	Filter flt;
	load_sub(flt, obj);
	obj.filters[name] = flt;
}

void PoeFilter::Loader::import(const string &fn)
{
	DataFile::load(obj, fn);
}

void PoeFilter::Loader::theme()
{
	load_sub(obj.theme);
}
